//#define SUPPORT_ERASE4K

#define SPI2_DDR	DDRD
#define SPI2_PORT 	PORTD
#define SPI2_PIN 	PIND
#define SPI2_SCK	PD6 // schime bug
#define SPI2_MOSI	PD3
#define SPI2_MISO	PD2

#define SPI2_CS_DDR	DDRC
#define SPI2_CS_PORT	PORTC
#define SPI2_CS		PC4

//#define HARD_SPI2

extern void init_spi2();
extern void term_spi2();

extern uint8_t spi2_write(uint8_t c);

#ifdef HARD_SPI2
static inline void spi2_send(uint8_t c) {
	UDR1 = c;
}

static inline uint8_t spi2_recv_send(uint8_t c) {
	uint8_t r;
	UDR1 = c;
	while (bit_is_clear(UCSR1A, RXC1))
		;
	r = UDR1;
	return r;
}

static inline uint8_t spi2_recv() {
	while (bit_is_clear(UCSR1A, RXC1))
		;
	return UDR1;
}
#endif

static inline uint8_t spi2_read(void) {
	return spi2_write(0x00);
}

static inline void spi2_enable(void) {
	bit_clear(SPI2_CS_PORT,SPI2_CS);
}

static inline void spi2_disable(void) {
	bit_set(SPI2_CS_PORT,SPI2_CS);
}

#define SERFLASH_TYPE_NONE		0x00
#define SERFLASH_TYPE_A25L080		0x42
#define	SERFLASH_TYPE_AT25DF041		0x81
#define	SERFLASH_TYPE_AT25DF081		0x82
#define	SERFLASH_TYPE_AT25DF161		0x84
#define	SERFLASH_TYPE_AT25DF321		0x88
#define	SERFLASH_TYPE_M25P20		0xc0
#define	SERFLASH_TYPE_M25P40		0xc1
#define	SERFLASH_TYPE_M25P80		0xc2

#define SERFLASH_RES		0xab
#define SERFLASH_RDID		0x9f
#define SERFLASH_READ		0x03
#define SERFLASH_RDSR		0x05
#define SERFLASH_WREN		0x06
#define SERFLASH_PAGE_PGM	0x02

#define SERFLASH_SECTOR_ERASE	0x20
#define SERFLASH_BLOCK_ERASE	0xd8
#define SERFLASH_CHIP_ERASE	0xc7

#define SERFLASH_STAT_WIP	(1<<0) // Write in Progress
#define SERFLASH_STAT_WEL	(1<<1) // Write Enable Latch
#define SERFLASH_STAT_SRWD	(1<<7) // Status Register Write Disable


extern uint8_t serflash_signature[3];
extern uint8_t serflash_type;
extern uint16_t serflash_size;

extern int8_t serflash_init(void);

extern void serflash_open(uint8_t cmd , int32_t offs);

static inline void serflash_close() {
	spi2_disable();
}

static inline uint8_t serflash_proc(uint8_t dat) {
	return spi2_write(dat);
}

static inline uint8_t serflash_status() {
	uint8_t ret;
	spi2_enable();
	spi2_write(SERFLASH_RDSR);
	ret = spi2_write(0);
	spi2_disable();
	return ret;
}

static inline void serflash_write_enable() {
	spi2_enable();
	spi2_write(SERFLASH_WREN);
	spi2_disable();
}

static inline void serflash_erase_4k(uint32_t offs) {

	serflash_write_enable();

	serflash_open(SERFLASH_SECTOR_ERASE,offs);
	serflash_close();
}

static inline void serflash_erase_64k(uint32_t offs) {

	serflash_write_enable();

	serflash_open(SERFLASH_BLOCK_ERASE,offs);
	serflash_close();
}
